@@ -27,7 +27,8 @@ from urllib.parse import urljoin
 
 import requests
 from cryptography.hazmat.primitives import serialization
-from pydantic import BaseModel, Field, validator
+from cryptography.hazmat.primitives.asymmetric import ec, rsa
+from pydantic import BaseModel, Field, StrictInt, StrictStr, validator
 
 DEFAULT_REKOR_URL = "https://rekor.sigstore.dev"
 STAGING_REKOR_URL = "https://rekor.sigstage.dev"
@@ -74,10 +75,10 @@ class RekorEntry:
 
 
 class RekorInclusionProof(BaseModel):
-    log_index: int = Field(..., alias="logIndex")
-    root_hash: str = Field(..., alias="rootHash")
-    tree_size: int = Field(..., alias="treeSize")
-    hashes: List[str] = Field(..., alias="hashes")
+    log_index: StrictInt = Field(..., alias="logIndex")
+    root_hash: StrictStr = Field(..., alias="rootHash")
+    tree_size: StrictInt = Field(..., alias="treeSize")
+    hashes: List[StrictStr] = Field(..., alias="hashes")
 
     class Config:
         allow_population_by_field_name = True
@@ -199,8 +200,27 @@ class RekorClient:
             {"Content-Type": "application/json", "Accept": "application/json"}
         )
 
-        self._pubkey = serialization.load_pem_public_key(pubkey)
-        self._ctfe_pubkey = serialization.load_pem_public_key(ctfe_pubkey)
+        pubkey = serialization.load_pem_public_key(pubkey)
+        if not isinstance(
+            pubkey,
+            ec.EllipticCurvePublicKey,
+        ):
+            raise RekorClientError(f"Invalid public key type: {pubkey}")
+        self._pubkey = pubkey
+
+        ctfe_pubkey = serialization.load_pem_public_key(ctfe_pubkey)
+        if not isinstance(
+            ctfe_pubkey,
+            (
+                rsa.RSAPublicKey,
+                ec.EllipticCurvePublicKey,
+            ),
+        ):
+            raise RekorClientError(f"Invalid CTFE public key type: {ctfe_pubkey}")
+        self._ctfe_pubkey = ctfe_pubkey
+
+    def __del__(self) -> None:
+        self.session.close()
 
     @classmethod
     def production(cls) -> RekorClient:
